package com.cultural.heritage.controller.order;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.order.OrderAddRequest;
import com.cultural.heritage.model.dto.order.OrderItemAddRequest;
import com.cultural.heritage.model.dto.order.OrderQueryRequest;
import com.cultural.heritage.model.dto.order.OrderUpdateRequest;
import com.cultural.heritage.model.entity.Order;
import com.cultural.heritage.model.entity.OrderItems;
import com.cultural.heritage.service.order.OrderItemService;
import com.cultural.heritage.service.order.OrderService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/order")
@Slf4j
@Tag(name = "订单管理模块")
@Transactional(rollbackFor = Exception.class)
public class OrderController {

    @Resource
    private OrderService orderService;



    @Resource
    private OrderItemService orderItemService;

//
//    /**
//     * 用户创建订单
//     * @param orderAddRequest  订单创建请求体
//     * @return  是否创建成功
//     */
//    @PostMapping("/add")
//    @Operation(summary = "小程序端创建用户订单", description = "参数：订单创建请求体，权限：所有人，方法名：addOrder")
//    public BaseResponse<Long> addOrder(@RequestBody OrderAddRequest orderAddRequest) {
//        if (orderAddRequest == null) {
//            throw new BusinessException(ErrorCode.PARAMS_ERROR);
//        }
//        Order order = new Order();
//        BeanUtils.copyProperties(orderAddRequest, order);
//        boolean save = orderService.save(order);
//        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);
//        return ResultUtils.success(order.getId());
//    }
//
//
//    /**
//     * 用户添加订单明细
//     * @param orderItemAddRequestList 订单明细添加请求体
//     * @return  是否添加成功
//     */
//    @PostMapping("/add/item")
//    @Operation(summary = "小程序端批量添加用户订单明细", description = "参数：订单明细添加请求体，权限：所有人，方法名：addOrderItem")
//    public BaseResponse<Boolean> addOrderItem(@RequestBody List<OrderItemAddRequest> orderItemAddRequestList) {
//        if (CollectionUtils.isEmpty(orderItemAddRequestList)) {
//            throw new BusinessException(ErrorCode.PARAMS_ERROR);
//        }
//        List<OrderItem> orderItems = orderItemAddRequestList.stream().map(orderItemAddRequest -> {
//            OrderItem orderItem = new OrderItem();
//            BeanUtils.copyProperties(orderItemAddRequest, orderItem);
//            return orderItem;
//        }).toList();
//        boolean result = orderItemService.saveBatch(orderItems);
//        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
//        return ResultUtils.success(true, "添加订单明细成功");
//    }


    /**
     * 用户创建订单
     * @param orderAddRequest 订单创建请求体
     * @return  是否创建成功
     */
    @PostMapping("/add")
    @Transactional(rollbackFor = Exception.class)
    @Operation(summary = "小程序端创建用户订单", description = "参数：订单创建请求体，排序字段 sortField: totalAmount，权限：所有人，方法名：addOrder")
    public BaseResponse<Boolean> addOrder(@RequestBody OrderAddRequest orderAddRequest) {
        if (orderAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 创建一个总订单
        Order order = new Order();
        BeanUtils.copyProperties(orderAddRequest, order);
        boolean result = orderService.save(order);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        // 创建订单明细
        Long id = order.getId();
        List<OrderItemAddRequest> orderItemList = orderAddRequest.getOrderItemList();
        List<OrderItems> newOrderItemsList = orderItemList.stream().map(orderItem -> {
            OrderItems orderItems = new OrderItems();
            BeanUtils.copyProperties(orderItem, orderItems);
            orderItems.setOrderId(id);
            return orderItems;
        }).toList();
        boolean save = orderItemService.saveBatch(newOrderItemsList);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * 分页查询所有订单
     * @param orderQueryRequest  订单查询请求体
     * @return 订单列表信息
     */
    @PostMapping("/list/page")
    @Operation(summary = "Web端管理员分页查询订单", description = "参数：订单查询请求体，权限：管理员(admin, boss)，方法名：listOrder")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<Order>> listOrder(@RequestBody OrderQueryRequest orderQueryRequest) {
        if (orderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = orderQueryRequest.getCurrent();
        long pageSize = orderQueryRequest.getPageSize();
        Page<Order> page = orderService.page(new Page<>(current, pageSize),
                orderService.getQueryWrapper(orderQueryRequest));
        return ResultUtils.success(page);
    }



    /**
     * 查询当前订单的订单明细
     * @param orderIdRequest 订单编号请求体
     * @return  订单明细列表信息
     */
    @PostMapping("/list/item")
    @Operation(summary = "Web端管理员查询订单明细", description = "参数：订单编号请求体，权限：管理员(admin, boss)，方法名：listOrderItem")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<List<OrderItems>> listOrderItem(@RequestBody CommonRequest orderIdRequest) {
        if (orderIdRequest == null || orderIdRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = orderIdRequest.getId();
        QueryWrapper<OrderItems> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderId", id);
        List<OrderItems> list = orderItemService.list(queryWrapper);
        return ResultUtils.success(list);
    }




    /**
     * 更新用户订单状态
     * @param orderUpdateRequest  订单状态更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "Web端管理员更新订单状态", description = "参数：订单状态更新请求体，权限：管理员(admin, boss)，方法名：updateOrderStatus")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateOrderStatus(@RequestBody OrderUpdateRequest orderUpdateRequest) {
        if (orderUpdateRequest == null || orderUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = orderUpdateRequest.getId();
        String orderStatus = orderUpdateRequest.getOrderStatus();
        UpdateWrapper<Order> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("orderStatus", orderStatus);
        boolean result = orderService.update(updateWrapper);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }


}
